# -*- coding: utf-8 -*-
"""
@author: Jose Manuel Martín Cabeza
"""

import StruPy2D as sp2

#---------------------------Barra aislada biempotrada Euler-------------------------------#

#------------------------------------2 elementos----------------------------------------#

"""Nodos"""
p1 = sp2.Node(0,0)
p2 = sp2.Node(4,0)
p3 = sp2.Node(8,0)

"""Materiales y secciones"""
m1 = sp2.Material('acero',2.1e11,7850)
s1 = sp2.Section('sec1',A=50e-4,I=200e-6,h=0.4)

"""Barras"""
b1 = sp2.Bar(1, p1, p2, m1, s1)
b2 = sp2.Bar(2, p2, p3, m1, s1)

"""Cargas"""
f1 = sp2.Load('f1','NOD',fx=-1)
p3.addLoad(f1)

"""Condiciones de contorno"""
p1.setBC(sp2.CLAMPED)
p3.setBC(sp2.H_MOV_CLA)

"""Generar estructura"""
S = sp2.Structure([b1, b2])
S.show()
S.setWeight('off')

"""Calculo estatico"""
St = sp2.Static(S, run=True)
St.show()
for B in St.bars:
    print(B)

"""Calculo pandeo global"""    
Bu = sp2.Buckling(St, run=True)
Bu.show()
print(Bu)

#------------------------------------4 elementos----------------------------------------#

''' 4 elementos '''

"""Nodos"""
p1 = sp2.Node(0,0)
p12 = sp2.Node(2,0)
p2 = sp2.Node(4,0)
p23 = sp2.Node(6,0)
p3 = sp2.Node(8,0)

"""Materiales y secciones"""
m1 = sp2.Material('acero',2.1e11,7850)
s1 = sp2.Section('sec1',A=50e-4,I=200e-6,h=0.4)

"""Barras"""
b1 = sp2.Bar(1, p1, p12, m1, s1)
b12 = sp2.Bar(1, p12, p2, m1, s1)
b2 = sp2.Bar(2, p2, p23, m1, s1)
b22 = sp2.Bar(2, p23, p3, m1, s1)

"""Cargas"""
f1 = sp2.Load('f1','NOD',fx=-1)
p3.addLoad(f1)

"""Condiciones de contorno"""
p1.setBC(sp2.CLAMPED)
p3.setBC(sp2.H_MOV_CLA)

"""Generar estructura"""
S = sp2.Structure([b1, b12, b2, b22])
S.show()
S.setWeight('off')

"""Calculo estatico"""
St = sp2.Static(S, run=True)
St.show()
for B in St.bars:
    print(B)
"""Calculo pandeo global"""    
Bu = sp2.Buckling(St, run=True)
Bu.show()
print(Bu)

#------------------------------------8 elementos----------------------------------------#

''' 8 elementos '''

"""Nodos"""
p1 = sp2.Node(0,0)
p112 = sp2.Node(1,0)
p12 = sp2.Node(2,0)
p122 = sp2.Node(3,0)
p2 = sp2.Node(4,0)
p223 = sp2.Node(5,0)
p23 = sp2.Node(6,0)
p233 = sp2.Node(7,0)
p3 = sp2.Node(8,0)

"""Materiales y secciones"""
m1 = sp2.Material('acero',2.1e11,7850)
s1 = sp2.Section('sec1',A=50e-4,I=200e-6,h=0.4)

"""Barras"""
b1 = sp2.Bar(1, p1, p112, m1, s1)
b2 = sp2.Bar(1, p112, p12, m1, s1)
b3 = sp2.Bar(2, p12, p122, m1, s1)
b4 = sp2.Bar(2, p122, p2, m1, s1)
b5 = sp2.Bar(2, p2, p223, m1, s1)
b6 = sp2.Bar(2, p223, p23, m1, s1)
b7 = sp2.Bar(2, p23, p233, m1, s1)
b8 = sp2.Bar(2, p233, p3, m1, s1)

"""Cargas"""
f1 = sp2.Load('f1','NOD',fx=-1)
p3.addLoad(f1)

"""Condiciones de contorno"""
p1.setBC(sp2.CLAMPED)
p3.setBC(sp2.H_MOV_CLA)

"""Generar estructura"""
S = sp2.Structure([b1, b2, b3, b4, b5, b6, b7, b8])
S.show()
S.setWeight('off')

"""Calculo estatico"""
St = sp2.Static(S, run=True)
St.show()
for B in St.bars:
    print(B)

"""Calculo pandeo global"""     
Bu = sp2.Buckling(St, run=True)
Bu.show()
print(Bu)
