        # -*- coding: utf-8 -*-
"""
Created on Tue Jun  1 12:26:01 2021

@author: Jose Manuel Martin Cabeza
"""
'''Importar librerias de calculo'''
import numpy as np
from math import sqrt, sin, cos
from matplotlib.axes._axes import Axes as mpAxes
import scipy.linalg as sp
import matplotlib.pyplot as plt

'''Importar scripts graficos'''
from StruPy2D.drawBC import drawBC, drawElasSup, drawDispSup
import StruPy2D.Graphics.Axes as spAxes
from StruPy2D.Graphics.graphNode import graphNode as gN
from StruPy2D.Graphics.graphBar import graphBar as gB
from StruPy2D.Graphics.Style import defNodeStyle, defBarStyle, defLoadStyle


class Buckling(): 
    def __init__(self,St,run=False,verModo=1):
        '''Comprobación de que St es un caso estatico''' 
        if(type(St).__name__ != "Static"):
            raise TypeError("First argument of Buckling constructor must be a Static object")
            
        self.St = St 
        self.S  = St.S 
        self.nds = St.S.nds          
        self.Kg = []
        self.NKGg = []
        self.cc = []
        self.verModo = verModo - 1
     
        if(run):
            self.run()
            self.ok = True
    
    def __str__(self):
            
        val = '\nSolution of buckling \n'
        i=0
        for i in range (len(self.D)):
            val = val + (r' Mode {}, lambda = {:1f}'.format(i+1,self.D[i])) +'\n'

        return (val)

    def run(self):
        """Ejecuta el calculo a pandeo de la estructura."""
     
        '''Recalcular las matrices K y N* KG con los axiles del cálculo estático'''
        self.recalculateKKGM()
        
        self.Kg = self.S.Kg
        self.NKGg = self.S.NKGg

        K = self.Kg
        NKG = self.NKGg
   
        '''Construccion de cdc en un array unidimensional''' 
        self.cdcVectorGen()
        

        '''Aplicación de las condiciones de contorno'''
        K = K[np.ix_(self.cc,self.cc)]
        NKG = NKG[np.ix_(self.cc,self.cc)]

        '''Resolución del problema de valores propios'''
        self.generateSolution(K,NKG)
        
        
    def recalculateKKGM(self):
        '''Obtiene K y N*KG global de la estructura '''
        barN = [] 

        '''Asigna la media del axil entre los valores de los extremos como propiedad a la barra'''
        for B in self.St.bars:
            barN.append((B.IF[0]-B.IF[3])/2)
            
        i=0
        for B in self.S.bars:
            B.nBu = float(barN[i])
            B.getGlobal()
            i = i + 1
        '''Reconstruye las matrices de la estructura en globales'''
        self.S.assembleKNKGM()
        
    def cdcVectorGen(self):
        '''Genera el vector de condiciones de contorno del problema'''
        for N in self.S.nds:
            for i in range (3):
                self.cc.append(N.bc[i])
        
        self.cc = np.array(self.cc)     
        '''Cambio a vector logico'''
        self.cc = self.cc==1
        

        
    def generateSolution(self, K, NKG):
        """Genera la solución del problema de autovalores del cálculo a pandeo"""

        try:
            [D,V] = sp.eig(K,NKG)
        except ValueError as err:
            err.args = ['No se ha encontrado ningún factor de carga.']
            raise err
        
        '''Considera los resultados positivos y finitos del problema de autovalores'''
        V = V[:,np.isfinite(D) & (D > 0)]
        D = np.real(D[np.isfinite(D) & (D > 0)])
        
        '''
        V = V[:,np.isfinite(D)]# Matriz de modos
        D = np.real(D[np.isfinite(D)])# Matriz de coeficientes
        '''

        d = np.sort(D)
        v = np.zeros(V.shape)
        for i in range(len(d)):
            v[:,i] = V[:,D == d[i]].reshape(len(v))
        
        self.D = d
        self.V = v
        
        """ Almacena los datos de pandeo global en cada nodo"""
        S = np.zeros(3*(len(self.S.nds)))
        S[self.cc] = self.V[:,self.verModo]
        
        i=0
        for N in self.S.nds: 
            N.ubug = S[3*i:3*i+3]
            i = i+1

        """Crea un pequeño GUI para estudiar la solución obtenida."""
    def show(self, axes = None, bc_scale = 0.04, k_scale = 0.1, f_scale = 0.075,
             showNodes = False, node_style = defNodeStyle, bar_style = defBarStyle,
             load_style = defLoadStyle):
        
            vM = self.verModo
       
            """Visualización de la estructura"""
            if(axes is None):
                ax = spAxes.SPAxes()
            elif(isinstance(axes, spAxes.SPAxes)):
                ax = axes
            elif(isinstance(axes, mpAxes)):
                pos = axes.get_position()
                ax = spAxes.SPAxes(axes.figure, [pos.x0,pos.y0,pos.width,pos.height])
            else:
                raise TypeError('Axes must be a matplotlib.pyplot.Axes or StruPy2D.Axes.SPAxes object.')
            
            """Cálculo del tamaño característico de la estructura."""
            x_list = self.S.nds.x()
            y_list = self.S.nds.y()
            tam = sqrt((min(y_list)-max(y_list))**2 + (min(x_list)-max(x_list))**2)
            
            """Aplicación de este factor a algunas dimensiones de los estilos"""
            load_style.dim = f_scale*tam if (load_style.dim is None) else load_style.dim
            
            """Visualización de nodos"""
            for N in self.S.nds:
               node_style.visible = showNodes
               ax.addElement(gN(N, node_style))
            
            """Visualización de barras con sus libertades"""
            for B in self.S.bars:
                ax.addElement(gB(B, bar_style))
                
            """Visualización de las condiciones de contorno y cargas nodales"""
            for N in self.S.nds:
                Lbc = bc_scale * tam
                Lk = k_scale * tam
                drawBC(ax,N,Lbc)
                
                if(True in [x!=0 for x in N.k]):
                    drawElasSup(ax,N,Lk)
                    
                if(True in [x != 0 for x in N.u]):
                    drawDispSup(ax,N,Lbc)          
            
            """Deformada de la estructura en el modo elegido"""
            self.deformateBuckling()
        
            plt.axis('equal')
            plt.title(r'Pandeo global, Modo {} con $\lambda_p = {:.2f}$'.format(vM+1,self.D[vM]))
                                          
            if(axes == None):
                return ax
            else:
                return None    

            
    def deformateBuckling(self):
                                 
            for B in self.S.bars:  

                '''matriz reducida de cambio de base'''
                Lred = np.matrix(np.array([[ cos(B.th), sin(B.th)],
                                        [-sin(B.th), cos(B.th)],]))
                
                '''  correccion de longitud de cada barra  '''              
                deltaUl = B.T*np.matrix([B.p1.ubug,B.p2.ubug]).reshape(6,1)
                Ldelta = B.L-float(deltaUl[0])+float(deltaUl[3])

                ''' solucion en coord locales de la deformada de la barra '''
                xx = np.linspace(0,Ldelta,21)
                
                aa = np.matrix([[0**3, 0**2, 0, 1],
                      [Ldelta**3, Ldelta**2, Ldelta, 1],
                      [3*0**2, 2*0, 1, 0],
                      [3*Ldelta**2, 2*Ldelta, 1, 0]])

                
                bb = np.matrix(np.array([[deltaUl[1]],
                                         [deltaUl[4]],
                                         [deltaUl[2]],
                                         [deltaUl[5]]])).reshape((len(aa),1))
                
                co = sp.solve(aa,bb)
                yy = co[0]*xx**3 + co[1]*xx**2 + co[2]*xx + co[3]
                
                XX = []
                YY = []
                
                ''' giro y desplazamiento de la solucion en coord locales al modelo global'''
                for j in range (len(xx)): 
                    a=float(B.p1.ubug[0])
                    b=float(B.p1.ubug[1])

                    xy = np.array([xx[j],yy[j]-yy[0]])
                    xy = np.linalg.inv(Lred)*xy.reshape(2,1)

                    XX.append(float(xy[0])+B.p1.x+a)
                    YY.append(float(xy[1])+B.p1.y+b)
            
                '''Representacion de la solucion en el grafico'''
                plt.plot(XX,YY,'r',lw=2)

                
             
            
        