# -*- coding: utf-8 -*-

from matplotlib.collections import PatchCollection
from matplotlib import patches as ptch
from matplotlib.path import Path as pth
from matplotlib.transforms import Affine2D

import numpy as np
from math import sin, cos, pi

class graphBar(PatchCollection):
    def __init__(self,bar, style, *args, **kwargs):
        patches = []
        self.bar = bar
        self.ax = None
        
        x0 = bar.p1.x
        y0 = bar.p1.y
        th = bar.th
        
        h = bar.sec.h if style.auto_scale else style.dim
        
        self.h = h # Guardamos el tamaño del canto dibujado
        
        x = x0 + 0.5*h*sin(th)
        y = y0 - 0.5*h*cos(th)
        
        patches.append(ptch.Rectangle((x,y), bar.L, h, angle = th*180/pi, 
                                  fc = style.FaceColor1,
                                  ec = style.EdgeColor1,
                                  lw = style.LineWidth1))
        
        # Releases
        ini = 0
        end = 0
        H = 2.5*h # Distancia entre centros de libertades
        k = style.rel_scale # Factor de escala para las libertades internas
        if(bar.releases[2]):
            ini += 1
            R = k*0.75*h
            
            xC = x + k*H*cos(th) - 0.5*h*sin(th)
            yC = y + 0.5*h*cos(th) + k*H*sin(th)
            
            patches.append(ptch.Circle((xC, yC), 1.5*R, ec = None, fc = 'w'))
            patches.append(ptch.Circle((xC, yC), R, ec = None, fc = style.FaceColor1))
            
        if(bar.releases[5]):
            end += 1
            R = k*0.75*h
            
            xC = x + bar.L*cos(th) - k*H*cos(th) - 0.5*h*sin(th)
            yC = y + bar.L*sin(th) + 0.5*h*cos(th) - k*H*sin(th)
            
            patches.append(ptch.Circle((xC, yC), 1.5*R, ec = None, fc = 'w'))
            patches.append(ptch.Circle((xC, yC), R, ec = None, fc = style.FaceColor1))
            
        if(bar.releases[1]):
            a = k*max(0.015*bar.L, h) # Ancho total del esquema
            e = (1+0.25*(k-1))*0.2*a # Espesor de las líneas perpendiculares
            R = 0.5*(a - 2*e) # Radio de las ruedas
            b = (1+0.25*(k-1))*max(0.03*bar.L, 2*h, 6*R) # Alto total del esquema
            c = 0.25*(b - 2*R) + R # Para las dos circunferencias exteriores
            
            xC = x + (ini+1)*H*cos(th) - 0.5*h*sin(th)
            yC = y + 0.5*h*cos(th) + (ini+1)*H*sin(th)
            x1 = xC - 0.5*a*cos(th) + 0.5*b*sin(th)
            y1 = yC - 0.5*b*cos(th) - 0.5*a*sin(th)
            x2 = x1 + e*cos(th) + 0.05*a*sin(th)
            y2 = y1 + e*sin(th) - 0.05*a*cos(th)
            
            patches.append(ptch.Rectangle((x1,y1), a, b, angle = th*180/pi, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Rectangle((x2,y2), a-2*e, 1.1*b, angle = th*180/pi, fc = 'w', ec = None))
            patches.append(ptch.Circle((xC, yC), 1.01*R, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Circle((xC + c*sin(th), yC - c*cos(th)), 1.01*R, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Circle((xC - c*sin(th), yC + c*cos(th)), 1.01*R, fc = style.FaceColor1, ec = None))
            ini += 1
            
        if(bar.releases[4]):
            a = k*max(0.015*bar.L, h) # Ancho total del esquema
            e = (1+0.25*(k-1))*0.2*a # Espesor de las líneas perpendiculares
            R = 0.5*(a - 2*e) # Radio de las ruedas
            b = (1+0.25*(k-1))*max(0.03*bar.L, 2*h, 6*R) # Alto total del esquema
            c = 0.25*(b - 2*R) + R # Para las dos circunferencias exteriores
            
            xC = x + bar.L*cos(th) - (end+1)*H*cos(th) - 0.5*h*sin(th)
            yC = y + bar.L*sin(th) + 0.5*h*cos(th) - (ini+1)*H*sin(th)
            x1 = xC - 0.5*a*cos(th) + 0.5*b*sin(th)
            y1 = yC - 0.5*b*cos(th) - 0.5*a*sin(th)
            x2 = x1 + e*cos(th) + 0.05*a*sin(th)
            y2 = y1 + e*sin(th) - 0.05*a*cos(th)
            
            patches.append(ptch.Rectangle((x1,y1), a, b, angle = th*180/pi, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Rectangle((x2,y2), a-2*e, 1.1*b, angle = th*180/pi, fc = 'w', ec = None))
            patches.append(ptch.Circle((xC, yC), 1.01*R, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Circle((xC + c*sin(th), yC - c*cos(th)), 1.01*R, fc = style.FaceColor1, ec = None))
            patches.append(ptch.Circle((xC - c*sin(th), yC + c*cos(th)), 1.01*R, fc = style.FaceColor1, ec = None))
            end += 1
            
        PatchCollection.__init__(self, patches, *args, **kwargs, 
                            match_original = True,
                            zorder = 1)
        self.set_visible(style.visible)